!---------------------------------------------------------------------------------------
! GRID_TO_NCDF: write_ncdf
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> write_ncdf
!!
!! Purpose:    Writes a 3D array to a ncdf file.
!!
!! Interface:
!!
!!    Inputs
!!             filename  -  name of the output file
!!             time      -  time dimension
!!             lon       -  longitude dimension
!!             lat       -  latitude dimension
!!             grid      -  the 3D floating variable
!!  
!---------------------------------------------------------------------------------------

subroutine write_ncdf(filename, time, grid)

  use mod_var
  use netcdf

  implicit none

  character(len=max_path_len),          intent(in) :: filename
  real(kind=8), dimension(ngrid),       intent(in) :: time
  real, dimension(nxgrid,nygrid,ngrid), intent(in) :: grid

  character(len=max_name_len) :: lonname, latname, timename, varname, varunit   
  integer                     :: nid, ierr, xid, yid, zid, ix, jy, kt
  integer                     :: lonid, latid, tid, varid
  integer, dimension(3)       :: dimids

  ! variable and dimension names
  lonname = 'longitude'
  latname = 'latitude'
  timename = 'time'
  varname = 'grid'
  varunit = 'sm3/kg'

  ! open netcdf file
  print*, 'Writing file: ',trim(filename)
  ierr=nf90_create(trim(filename),nf90_CLOBBER,nid)
  if(ierr.gt.0) then
    print*, 'ERROR: cannot create file: ',trim(filename)
  endif

  ! define dimensions
  ierr=nf90_def_dim(nid,lonname,nxgrid,xid)
  ierr=nf90_def_dim(nid,latname,nygrid,yid)
  ierr=nf90_def_dim(nid,timename,ngrid,zid)
  ierr=nf90_enddef(nid)

  ! define dimension variables
  ierr=nf90_redef(nid)
  ierr=nf90_def_var(nid,lonname,nf90_real,xid,varid)
  ierr=nf90_put_att(nid,varid,'title',trim(lonname))
  ierr=nf90_enddef(nid)
  ierr=nf90_put_var(nid,varid,glon)
  ierr=nf90_redef(nid)
  ierr=nf90_def_var(nid,latname,nf90_real,yid,varid)
  ierr=nf90_put_att(nid,varid,'title',trim(latname))
  ierr=nf90_enddef(nid)
  ierr=nf90_put_var(nid,varid,glat)
  ierr=nf90_redef(nid)
  ierr=nf90_def_var(nid,timename,nf90_double,zid,varid)
  ierr=nf90_put_att(nid,varid,'title',trim(timename))
  ierr=nf90_enddef(nid)
  ierr=nf90_put_var(nid,varid,time)

  ! define dependent variable
  dimids(:)=(/xid,yid,zid/)
  ierr=nf90_redef(nid)
  ierr=nf90_def_var(nid,varname,nf90_real,dimids,varid)
  ierr=nf90_put_att(nid,varid,'title',trim(varname))
  ierr=nf90_put_att(nid,varid,'unit',trim(varunit))
  ierr=nf90_enddef(nid)

  ! insert variable
  ierr=nf90_put_var(nid,varid,grid)

  ierr=nf90_close(nid)

end subroutine write_ncdf
